var versionNumber = '1.3.1';
var versionRequest;
var defaultPreCode = false;
var actualPreCode = defaultPreCode;
var defaultMode = 'NCR';
var actualMode;

// Create unique key for each instance of widget
function createkey(key) {
  return widget.identifier + '-' + key;
}

function load() {
  setupParts();
  clearInput();
  document.getElementById('versionNumber').innerText = versionNumber;
  // Try to retrieve preferences, if they exist
  if(window.widget) {
    actualPreCode = widget.preferenceForKey(createkey('precode'));
    actualMode = widget.preferenceForKey(createkey('mode'));
  }

  if(actualPreCode === null) {
    actualPreCode = defaultPreCode;
    widget.setPreferenceForKey(actualPreCode,createkey('precode'));
  }

  if(actualMode === null) {
    actualMode = defaultMode;
    widget.setPreferenceForKey(actualMode,createkey('mode'));
  }
}

function remove() {
  widget.setPreferenceForKey(null,createkey('precode'));
  widget.setPreferenceForKey(null,createkey('mode'));
}

function hide() {
  // your widget has just been hidden stop any timers to
  // prevent cpu usage
}

function show() {
  var userInput = document.getElementById('userInput');
  userInput.focus();
  userInput.select();
}

function showBack(event) {
  var front = document.getElementById('front');
  var back = document.getElementById('back');

  if (window.widget)
    widget.prepareForTransition('ToBack');

  if (actualPreCode)
    document.getElementById('precode').checked = true;
  else
    document.getElementById('precode').checked = false;

  if (actualMode == 'NCR')
    document.getElementById('mode').checked = true;
  else
    document.getElementById('mode').checked = false;

  front.style.display='none';
  back.style.display='block';

  if (window.widget)
    setTimeout('widget.performTransition();', 0);
  checkForNewVersion();
}

function showFront(event) {
  var front = document.getElementById('front');
  var back = document.getElementById('back');

  if (window.widget)
    widget.prepareForTransition('ToFront');

  front.style.display='block';
  back.style.display='none';

  if (window.widget)
    setTimeout('widget.performTransition();', 0);
}

if (window.widget) {
  widget.onremove = remove;
  widget.onhide = hide;
  widget.onshow = show;
}

function checkInput(event) {
  if (document.getElementById('userInput').value === '') {
    document.getElementById('indicator').object.setValue(0);
    document.getElementById('cut').style.display = 'none';
  }
}

function clearInput(event) {
  var userInput = document.getElementById('userInput');
  userInput.value = '';
  userInput.focus();
  userInput.select();
  document.getElementById('indicator').object.setValue(0);
  document.getElementById('cut').style.display = 'none';
}

function convert(event) {
  var userInput = document.getElementById('userInput');
  if (userInput.value !== '') {
    document.getElementById('indicator').object.setValue(1);
    var userInputCode = userInput.value;
    var convertedText = htmlEntities(userInputCode, actualMode);
    if (actualPreCode) {
      convertedText = '<pre><code>' + convertedText + '</code></pre>';
    }
    userInput.value = convertedText;
    document.getElementById('indicator').object.setValue(2);
    document.getElementById('cut').style.display = 'block';
  }
}

function htmlEntities(str, mod) {
  var chars = ['&', '¡', '¢', '£', '¤', '¥', '¦', '§', '¨', '©', 
  'ª', '«', '¬', '®', '¯', '°', '±', '²', '³', '´', 
  'µ', '¶', '·', '¸', '¹', 'º', '»', '¼', '½', '¾', 
  '¿', 'À', 'Á', 'Â', 'Ã', 'Ä', 'Å', 'Æ', 'Ç', 'È', 
  'É', 'Ê', 'Ë', 'Ì', 'Í', 'Î', 'Ï', 'Ð', 'Ñ', 'Ò', 
  'Ó', 'Ô', 'Õ', 'Ö', '×', 'Ø', 'Ù', 'Ú', 'Û', 'Ü', 
  'Ý', 'Þ', 'ß', 'à', 'á', 'â', 'ã', 'ä', 'å', 'æ', 
  'ç', 'è', 'é', 'ê', 'ë', 'ì', 'í', 'î', 'ï', 'ð', 
  'ñ', 'ò', 'ó', 'ô', 'õ', 'ö', '÷', 'ø', 'ù', 'ú', 
  'û', 'ü', 'ý', 'þ', 'ÿ', 'ƒ', 'Α', 'Β', 'Γ', 'Δ', 
  'Ε', 'Ζ', 'Η', 'Θ', 'Ι', 'Κ', 'Λ', 'Μ', 'Ν', 'Ξ', 
  'Ο', 'Π', 'Ρ', 'Σ', 'Τ', 'Υ', 'Φ', 'Χ', 'Ψ', 'Ω', 
  'α', 'β', 'γ', 'δ', 'ε', 'ζ', 'η', 'θ', 'ι', 'κ', 
  'λ', 'μ', 'ν', 'ξ', 'ο', 'π', 'ρ', 'ς', 'σ', 'τ', 
  'υ', 'φ', 'χ', 'ψ', 'ω', 'ϑ', 'ϒ', 'ϖ', '•', '…', 
  '′', '″', '‾', '⁄', '℘', 'ℑ', 'ℜ', '™', 'ℵ', '←', 
  '↑', '→', '↓', '↔', '↵', '⇐', '⇒', '⇔', '∀', '∂', 
  '∃', '∅', '∇', '∈', '∉', '∋', '∏', '∑', '−', '∗', 
  '√', '∝', '∞', '∠', '∧', '∨', '∩', '∪', '∫', '∴', 
  '∼', '≅', '≈', '≠', '≡', '≤', '≥', '⊂', '⊃', '⊄', 
  '⊆', '⊇', '⊕', '⊗', '⊥', '⋅', '⌈', '⌉', '⌊', '⌋', 
  '◊', '♠', '♣', '♥', '♦', '"', '<', '>', 'Œ', 'œ', 
  'Š', 'š', 'Ÿ', 'ˆ', '˜', '–', '—', '‘', '’', '‚', 
  '“', '”', '„', '†', '‡', '‰', '‹', '›', '€'];

  var character_entities = ['amp', 'iexcl', 'cent', 'pound', 'curren', 'yen', 'brvbar', 'sect', 'uml', 'copy', 
  'ordf', 'laquo', 'not', 'reg', 'macr', 'deg', 'plusmn', 'sup2', 'sup3', 'acute', 
  'micro', 'para', 'middot', 'cedil', 'sup1', 'ordm', 'raquo', 'frac14', 'frac12', 'frac34', 
  'iquest', 'Agrave', 'Aacute', 'Acirc', 'Atilde', 'Auml', 'Aring', 'AElig', 'Ccedil', 'Egrave', 
  'Eacute', 'Ecirc', 'Euml', 'Igrave', 'Iacute', 'Icirc', 'Iuml', 'ETH', 'Ntilde', 'Ograve', 
  'Oacute', 'Ocirc', 'Otilde', 'Ouml', 'times', 'Oslash', 'Ugrave', 'Uacute', 'Ucirc', 'Uuml', 
  'Yacute', 'THORN', 'szlig', 'agrave', 'aacute', 'acirc', 'atilde', 'auml', 'aring', 'aelig', 
  'ccedil', 'egrave', 'eacute', 'ecirc', 'euml', 'igrave', 'iacute', 'icirc', 'iuml', 'eth', 
  'ntilde', 'ograve', 'oacute', 'ocirc', 'otilde', 'ouml', 'divide', 'oslash', 'ugrave', 'uacute', 
  'ucirc', 'uuml', 'yacute', 'thorn', 'yuml', 'fnof', 'Alpha', 'Beta', 'Gamma', 'Delta', 
  'Epsilon', 'Zeta', 'Eta', 'Theta', 'Iota', 'Kappa', 'Lambda', 'Mu', 'Nu', 'Xi', 
  'Omicron', 'Pi', 'Rho', 'Sigma', 'Tau', 'Upsilon', 'Phi', 'Chi', 'Psi', 'Omega', 
  'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta', 'eta', 'theta', 'iota', 'kappa', 
  'lambda', 'mu', 'nu', 'xi', 'omicron', 'pi', 'rho', 'sigmaf', 'sigma', 'tau', 
  'upsilon', 'phi', 'chi', 'psi', 'omega', 'thetasym', 'upsih', 'piv', 'bull', 'hellip', 
  'prime', 'Prime', 'oline', 'frasl', 'weierp', 'image', 'real', 'trade', 'alefsym', 'larr', 
  'uarr', 'rarr', 'darr', 'harr', 'crarr', 'lArr', 'rArr', 'hArr', 'forall', 'part', 
  'exist', 'empty', 'nabla', 'isin', 'notin', 'ni', 'prod', 'sum', 'minus', 'lowast', 
  'radic', 'prop', 'infin', 'ang', 'and', 'or', 'cap', 'cup', 'int', 'there4', 
  'sim', 'cong', 'asymp', 'ne', 'equiv', 'le', 'ge', 'sub', 'sup', 'nsub', 
  'sube', 'supe', 'oplus', 'otimes', 'perp', 'sdot', 'lceil', 'rceil', 'lfloor', 'rfloor', 
  'loz', 'spades', 'clubs', 'hearts', 'diams', 'quot', 'lt', 'gt', 'OElig', 'oelig', 
  'Scaron', 'scaron', 'Yuml', 'circ', 'tilde', 'ndash', 'mdash', 'lsquo', 'rsquo', 'sbquo', 
  'ldquo', 'rdquo', 'bdquo', 'dagger', 'Dagger', 'permil', 'lsaquo', 'rsaquo', 'euro'];

  var code_point = ['38', '161', '162', '163', '164', '165', '166', '167', '168', '169', 
  '170', '171', '172', '174', '175', '176', '177', '178', '179', '180', 
  '181', '182', '183', '184', '185', '186', '187', '188', '189', '190', 
  '191', '192', '193', '194', '195', '196', '197', '198', '199', '200', 
  '201', '202', '203', '204', '205', '206', '207', '208', '209', '210', 
  '211', '212', '213', '214', '215', '216', '217', '218', '219', '220', 
  '221', '222', '223', '224', '225', '226', '227', '228', '229', '230', 
  '231', '232', '233', '234', '235', '236', '237', '238', '239', '240', 
  '241', '242', '243', '244', '245', '246', '247', '248', '249', '250', 
  '251', '252', '253', '254', '255', '402', '913', '914', '915', '916', 
  '917', '918', '919', '920', '921', '922', '923', '924', '925', '926', 
  '927', '928', '929', '931', '932', '933', '934', '935', '936', '937', 
  '945', '946', '947', '948', '949', '950', '951', '952', '953', '954', 
  '955', '956', '957', '958', '959', '960', '961', '962', '963', '964', 
  '965', '966', '967', '968', '969', '977', '978', '982', '8226', '8230', 
  '8242', '8243', '8254', '8260', '8472', '8465', '8476', '8482', '8501', '8592', 
  '8593', '8594', '8595', '8596', '8629', '8656', '8658', '8660', '8704', '8706', 
  '8707', '8709', '8711', '8712', '8713', '8715', '8719', '8721', '8722', '8727', 
  '8730', '8733', '8734', '8736', '8743', '8744', '8745', '8746', '8747', '8756', 
  '8764', '8773', '8776', '8800', '8801', '8804', '8805', '8834', '8835', '8836', 
  '8838', '8839', '8853', '8855', '8869', '8901', '8968', '8969', '8970', '8971', 
  '9674', '9824', '9827', '9829', '9830', '34', '60', '62', '338', '339', 
  '352', '353', '376', '710', '732', '8211', '8212', '8216', '8217', '8218', 
  '8220', '8221', '8222', '8224', '8225', '8240', '8249', '8250', '8364'];					

  var h, i, j, k, convertingCode, escapeCode, matchString, convertedString = '';
  if (mod == 'NCR') {
    convertingCode = code_point;
    escapeCode = '&#';
  }
  else {
    convertingCode = character_entities;
    escapeCode = '&';
  }
  for (h = 0, i = str.length ; h < i; h++) {
    matchString = '';
    for (j = 0, k = chars.length ; j < k; j++) {
      if (str[h].match(chars[j])) {
        matchString = str[h].replace(chars[j], escapeCode + convertingCode[j] + ';');
        break;
      }
    }
    if (matchString !== '')
      convertedString += matchString;
    else
      convertedString += str[h];
  }
  return convertedString;
}

function copy(event) {
  var userInputText = document.getElementById('userInput').value;
  if (userInputText !== '')
    copyToPasteboard(userInputText);
  document.getElementById('cut').src = 'images/cut.png';
}

function copyToPasteboard(inputText) {
  if (window.widget) widget.system("/bin/echo -n '" + inputText.replace(/\'/g,'&#39;') +"' | /usr/bin/pbcopy", null);
}

function cutClicked(event) {
  document.getElementById('cut').src = 'images/cut_red.png';
}

function goHome(event) {
  widget.openURL('http://appletree.or.kr/widgets/CodePostable/');
}

// Download version.txt from website and check version numbers
function checkForNewVersion() {
  var versionURL = 'http://appletree.or.kr/widgets/CodePostable/version.txt';

  if (versionRequest)
    versionRequest.abort();

  versionRequest = new XMLHttpRequest();
  versionRequest.onreadystatechange = getVersionNumber;
  versionRequest.open('GET', versionURL, true);
  versionRequest.send(null);
}

// If new version is available, display link and notice
function getVersionNumber() {
  if (versionRequest.readyState == 4 && versionRequest.status == 200) {
    if (versionRequest.responseText != versionNumber) {			
      document.getElementById('newVersion').style.display = 'block';
      document.getElementById('backDevelopedWith').style.display = 'none';
    }
    else {
      document.getElementById('newVersion').style.display = 'none';
      document.getElementById('backDevelopedWith').style.display = 'blick';
    }
    versionRequest = null;
  } 
  else if (versionRequest.readyState == 4) {
    versionRequest.abort();
    versionRequest = null;
  }
}

function precodeChanged(event) {
  if (document.getElementById('precode').checked)
    actualPreCode = true;
  else
    actualPreCode = false;
  widget.setPreferenceForKey(actualPreCode,createkey('precode'));
}

function modeChanged(event) {
  if (document.getElementById('mode').checked)
    actualMode = 'NCR';
  else
    actualMode = 'CER';
  widget.setPreferenceForKey(actualMode,createkey('mode'));
}
